@@ -23,6 +23,7 @@
 #include "gailutil.h"
 #include "gailtoplevel.h"
 #include "gailwindow.h"
+#include "gail-private-macros.h"
 
 static void		gail_util_class_init			(GailUtilClass		*klass);
 
@@ -168,7 +169,6 @@ gail_util_add_global_event_listener (GSignalEmissionHook listener,
       g_strfreev (split_string);
     }
 
-  g_free (split_string);
   return rc;
 }
 
@@ -433,7 +433,7 @@ state_event_watcher (GSignalInvocationHint  *hint,
     return FALSE;
 
   event = g_value_get_boxed (param_values + 1);
-  g_return_val_if_fail (event->type == GDK_WINDOW_STATE, FALSE);
+  gail_return_val_if_fail (event->type == GDK_WINDOW_STATE, FALSE);
   widget = GTK_WIDGET (object);
 
   if (event->new_window_state & GDK_WINDOW_STATE_MAXIMIZED)
@@ -452,15 +452,22 @@ state_event_watcher (GSignalInvocationHint  *hint,
     return TRUE;
   
   atk_obj = gtk_widget_get_accessible (widget);
-  g_return_val_if_fail (GAIL_IS_WINDOW (atk_obj), FALSE);
-  parent = atk_object_get_parent (atk_obj);
-  if (parent == atk_get_root ())
-    {
-      signal_id = g_signal_lookup (signal_name, GAIL_TYPE_WINDOW); 
-      g_signal_emit (atk_obj, signal_id, 0);
-    }
 
-  return TRUE;
+  if (GAIL_IS_WINDOW (atk_obj))
+    {
+      parent = atk_object_get_parent (atk_obj);
+      if (parent == atk_get_root ())
+	{
+	  signal_id = g_signal_lookup (signal_name, GAIL_TYPE_WINDOW); 
+	  g_signal_emit (atk_obj, signal_id, 0);
+	}
+      
+      return TRUE;
+    }
+  else
+    {
+      return FALSE;
+    }
 }
 
 static void
@@ -470,10 +477,10 @@ window_added (AtkObject *atk_obj,
 {
   GtkWidget *widget;
 
-  g_return_if_fail (GAIL_IS_WINDOW (child));
+  if (!GAIL_IS_WINDOW (child)) return;
 
   widget = GTK_ACCESSIBLE (child)->widget;
-  g_return_if_fail (widget);
+  gail_return_if_fail (widget);
 
   g_signal_connect (widget, "focus-in-event",  
                     (GCallback) window_focus, NULL);
@@ -491,10 +498,10 @@ window_removed (AtkObject *atk_obj,
   GtkWidget *widget;
   GtkWindow *window;
 
-  g_return_if_fail (GAIL_IS_WINDOW (child));
+  if (!GAIL_IS_WINDOW (child)) return;
 
   widget = GTK_ACCESSIBLE (child)->widget;
-  g_return_if_fail (widget);
+  gail_return_if_fail (widget);
 
   window = GTK_WINDOW (widget);
   /*
@@ -579,13 +586,19 @@ configure_event_watcher (GSignalInvocationHint  *hint,
     }
 
   atk_obj = gtk_widget_get_accessible (widget);
-  g_return_val_if_fail (GAIL_IS_WINDOW (atk_obj), FALSE);
-  parent = atk_object_get_parent (atk_obj);
-  if (parent == atk_get_root ())
+  if (GAIL_IS_WINDOW (atk_obj))
     {
-      signal_id = g_signal_lookup (signal_name, GAIL_TYPE_WINDOW); 
-      g_signal_emit (atk_obj, signal_id, 0);
+      parent = atk_object_get_parent (atk_obj);
+      if (parent == atk_get_root ())
+	{
+	  signal_id = g_signal_lookup (signal_name, GAIL_TYPE_WINDOW); 
+	  g_signal_emit (atk_obj, signal_id, 0);
+	}
+      
+      return TRUE;
+    }
+  else
+    {
+      return FALSE;
     }
-
-  return TRUE;
 }
