@@ -21,6 +21,7 @@
 #include <gtk/gtk.h>
 #include "gailwindow.h"
 #include "gailtoplevel.h"
+#include "gail-private-macros.h"
 
 enum {
   ACTIVATE,
@@ -29,6 +30,7 @@ enum {
   DESTROY,
   MAXIMIZE,
   MINIMIZE,
+  MOVE,
   RESIZE,
   RESTORE,
   LAST_SIGNAL
@@ -180,6 +182,14 @@ gail_window_class_init (GailWindowClass *klass)
                   NULL, NULL,
                   g_cclosure_marshal_VOID__VOID,
                   G_TYPE_NONE, 0);
+  gail_window_signals [MOVE] =
+    g_signal_new ("move",
+                  G_TYPE_FROM_CLASS (klass),
+                  G_SIGNAL_RUN_LAST,
+                  0, /* default signal handler */
+                  NULL, NULL,
+                  g_cclosure_marshal_VOID__VOID,
+                  G_TYPE_NONE, 0);
   gail_window_signals [RESIZE] =
     g_signal_new ("resize",
                   G_TYPE_FROM_CLASS (klass),
@@ -204,13 +214,13 @@ gail_window_new (GtkWidget *widget)
   GObject *object;
   AtkObject *accessible;
 
-  g_return_val_if_fail (widget != NULL, NULL);
+  gail_return_val_if_fail (widget != NULL, NULL);
   /*
    * A GailWindow can be created for a GtkHandleBox or a GtkWindow
    */
   if (!GTK_IS_WINDOW (widget) &&
       !GTK_IS_HANDLE_BOX (widget))
-    g_return_val_if_fail (FALSE, NULL);
+    gail_return_val_if_fail (FALSE, NULL);
 
   object = g_object_new (GAIL_TYPE_WINDOW, NULL);
 
@@ -312,7 +322,7 @@ gail_window_get_name (AtkObject *accessible)
          */
         return NULL;
 
-      g_return_val_if_fail (GTK_IS_WIDGET (widget), NULL);
+      gail_return_val_if_fail (GTK_IS_WIDGET (widget), NULL);
 
       if (GTK_IS_WINDOW (widget))
         {
@@ -325,7 +335,12 @@ gail_window_get_name (AtkObject *accessible)
               GtkWidget *child;
 
               child = gtk_bin_get_child (GTK_BIN (window));
-              g_return_val_if_fail (GTK_IS_LABEL (child), NULL);
+	      /* could be some kind of egg notification bubble thingy? */
+              if (!GTK_IS_LABEL (child)) 
+	      { 
+		  g_message ("ATK_ROLE_TOOLTIP object found, but doesn't look like a tooltip.");
+		  return NULL;
+	      }
               name = gtk_label_get_text (GTK_LABEL (child));
             }
         }
@@ -348,7 +363,6 @@ gail_window_get_index_in_parent (AtkObject *accessible)
 {
   GtkWidget* widget = GTK_ACCESSIBLE (accessible)->widget; 
   AtkObject* atk_obj = atk_get_root ();
-  GailToplevel* toplevel = GAIL_TOPLEVEL (atk_obj);
   gint index = -1;
 
   if (widget == NULL)
@@ -357,7 +371,7 @@ gail_window_get_index_in_parent (AtkObject *accessible)
      */
     return -1;
 
-  g_return_val_if_fail (GTK_IS_WIDGET (widget), -1);
+  gail_return_val_if_fail (GTK_IS_WIDGET (widget), -1);
 
   index = ATK_OBJECT_CLASS (parent_class)->get_index_in_parent (accessible);
   if (index != -1)
@@ -366,8 +380,21 @@ gail_window_get_index_in_parent (AtkObject *accessible)
   if (GTK_IS_WINDOW (widget))
     {
       GtkWindow *window = GTK_WINDOW (widget);
-
-      index = g_list_index (toplevel->window_list, window);
+      if (GAIL_IS_TOPLEVEL (atk_obj))
+        {
+	  GailToplevel* toplevel = GAIL_TOPLEVEL (atk_obj);
+	  index = g_list_index (toplevel->window_list, window);
+	}
+      else
+        {
+	  int i, sibling_count = atk_object_get_n_accessible_children (atk_obj);
+	  for (i = 0; i < sibling_count && index == -1; ++i)
+	    {
+	      AtkObject *child = atk_object_ref_accessible_child (atk_obj, i);
+	      if (accessible == child) index = i;
+	      g_object_unref (G_OBJECT (child));
+	    }
+	}
     }
   return index;
 }
@@ -389,13 +416,12 @@ gail_window_ref_relation_set (AtkObject *obj)
 {
   GtkWidget *widget;
   AtkRelationSet *relation_set;
-  GtkWidget *label;
   AtkObject *array[1];
   AtkRelation* relation;
   GtkWidget *current_widget;
   gboolean ret;
 
-  g_return_val_if_fail (GAIL_IS_WIDGET (obj), NULL);
+  gail_return_val_if_fail (GAIL_IS_WIDGET (obj), NULL);
 
   widget = GTK_ACCESSIBLE (obj)->widget;
   if (widget == NULL)
@@ -467,10 +493,15 @@ idle_notify_name_change (gpointer data)
   GailWindow *window;
   AtkObject *obj;
 
+  GDK_THREADS_ENTER ();
+
   window = GAIL_WINDOW (data);
   window->name_change_handler = 0;
   if (GTK_ACCESSIBLE (window)->widget == NULL)
-    return FALSE;
+    {
+      GDK_THREADS_LEAVE ();
+      return FALSE;
+    }
 
   obj = ATK_OBJECT (window);
   if (obj->name == NULL)
@@ -481,6 +512,7 @@ idle_notify_name_change (gpointer data)
       g_object_notify (G_OBJECT (obj), "accessible-name");
     }
   g_signal_emit_by_name (obj, "visible_data_changed");
+  GDK_THREADS_LEAVE ();
   return FALSE;
 }
 
@@ -534,7 +566,7 @@ gail_window_state_event_gtk (GtkWidget           *widget,
 static void
 atk_component_interface_init (AtkComponentIface *iface)
 {
-  g_return_if_fail (iface != NULL);
+  gail_return_if_fail (iface != NULL);
 
   iface->get_extents = gail_window_get_extents;
   iface->get_size = gail_window_get_size;
@@ -559,7 +591,7 @@ gail_window_get_extents (AtkComponent  *component,
      */
     return;
 
-  g_return_if_fail (GTK_IS_WINDOW (widget));
+  gail_return_if_fail (GTK_IS_WINDOW (widget));
 
   if (!GTK_WIDGET_TOPLEVEL (widget))
     {
@@ -604,7 +636,7 @@ gail_window_get_size (AtkComponent *component,
      */
     return;
 
-  g_return_if_fail (GTK_IS_WINDOW (widget));
+  gail_return_if_fail (GTK_IS_WINDOW (widget));
 
   if (!GTK_WIDGET_TOPLEVEL (widget))
     {
@@ -620,6 +652,8 @@ gail_window_get_size (AtkComponent *component,
   *height = rect.height;
 }
 
+#if defined (GDK_WINDOWING_X11)
+
 #include <X11/Xlib.h>
 #include <X11/Xatom.h>
 #include <gdk/gdkx.h>
@@ -651,7 +685,7 @@ get_window_desktop (Window window)
   int             format;
   gulong          nitems;
   gulong          bytes_after;
-  gulong         *cardinals;
+  guchar         *cardinals;
   int             error;
   int             result;
   int             desktop;
@@ -665,7 +699,7 @@ get_window_desktop (Window window)
                                0, G_MAXLONG,
                                False, XA_CARDINAL,
                                &ret_type, &format, &nitems,
-                               &bytes_after, (guchar **) &cardinals);
+                               &bytes_after, &cardinals);
   error = gdk_error_trap_pop();
   /* nitems < 1 will occur if the property is not set */
   if (error != Success || result != Success || nitems < 1)
@@ -702,7 +736,7 @@ get_stacked_windows (GailScreenInfo *info)
   int     format;
   gulong  nitems;
   gulong  bytes_after;
-  Window *data;
+  guchar *data;
   int     error;
   int     result;
   int     i;
@@ -721,7 +755,7 @@ get_stacked_windows (GailScreenInfo *info)
                                _net_client_list_stacking,
                                0, G_MAXLONG,
                                False, XA_WINDOW, &ret_type, &format, &nitems,
-                               &bytes_after, (guchar **)&data);
+                               &bytes_after, &data);
   error = gdk_error_trap_pop ();
   /* nitems < 1 will occur if the property is not set */
   if (error != Success || result != Success || nitems < 1)
@@ -760,7 +794,7 @@ get_stacked_windows (GailScreenInfo *info)
         }
     }
   free_screen_info (info);
-  info->stacked_windows = data;
+  info->stacked_windows = (Window*) data;
   info->stacked_windows_len = nitems;
   info->desktop = desktops;
   info->desktop_changed = desktops_changed;
@@ -773,11 +807,15 @@ update_screen_info (gpointer data)
 {
   int screen_n = GPOINTER_TO_INT (data);
 
+  GDK_THREADS_ENTER ();
+
   gail_screens [screen_n].update_handler = 0;
   gail_screens [screen_n].update_stacked_windows = FALSE;
 
   get_stacked_windows (&gail_screens [screen_n]);
 
+  GDK_THREADS_LEAVE ();
+
   return FALSE;
 }
 
@@ -788,6 +826,8 @@ update_desktop_info (gpointer data)
   GailScreenInfo *info;
   int i;
 
+  GDK_THREADS_ENTER ();
+
   info = &gail_screens [screen_n];
   info->update_desktop_handler = 0;
 
@@ -800,6 +840,8 @@ update_desktop_info (gpointer data)
         }
     }
 
+  GDK_THREADS_LEAVE ();
+
   return FALSE;
 }
 
@@ -929,7 +971,7 @@ get_screen_info (GdkScreen *screen)
 {
   int screen_n;
 
-  g_return_val_if_fail (GDK_IS_SCREEN (screen), NULL);
+  gail_return_val_if_fail (GDK_IS_SCREEN (screen), NULL);
 
   screen_n = gdk_screen_get_number (screen);
 
@@ -957,12 +999,12 @@ get_window_zorder (GdkWindow *window)
   int             zorder;
   int             w_desktop;
 
-  g_return_val_if_fail (GDK_IS_WINDOW (window), -1);
+  gail_return_val_if_fail (GDK_IS_WINDOW (window), -1);
 
   info = get_screen_info (
 		gdk_drawable_get_screen (GDK_DRAWABLE (window)));
 
-  g_return_val_if_fail (info->stacked_windows != NULL, -1);
+  gail_return_val_if_fail (info->stacked_windows != NULL, -1);
 
   xid = GDK_WINDOW_XID (window);
 
@@ -1006,7 +1048,29 @@ gail_window_get_mdi_zorder (AtkComponent *component)
      */
     return -1;
 
-  g_return_val_if_fail (GTK_IS_WINDOW (widget), -1);
+  gail_return_val_if_fail (GTK_IS_WINDOW (widget), -1);
 
   return get_window_zorder (widget->window);
 }
+
+#elif defined (GDK_WINDOWING_WIN32)
+
+static gint
+gail_window_get_mdi_zorder (AtkComponent *component)
+{
+  GtkWidget *widget = GTK_ACCESSIBLE (component)->widget;
+
+  if (widget == NULL)
+    /*
+     * State is defunct
+     */
+    return -1;
+
+  gail_return_val_if_fail (GTK_IS_WINDOW (widget), -1);
+
+  return 0;			/* Punt, FIXME */
+}
+
+#else
+#error Port to this GDK backend
+#endif
